import React, {
	createContext,
	FC,
	MutableRefObject,
	ReactNode,
	useContext,
	useEffect,
	useMemo,
	useRef,
	useState,
} from 'react';
import { omit } from 'ramda';
import { DeviceEventEmitter, StyleProp, View, ViewStyle } from 'react-native';

export type Modal<P> = {
	Component?: FC<P>;
	id: string;
	props?: Partial<P>;
	style?: {
		component?: StyleProp<ViewStyle>;
		overflow?: StyleProp<ViewStyle>;
	};
};

export type ModalId = Modal<any>['id'];

export type ModalContextType = {
	close: (modalId: ModalId) => void;
	open: <P>(modalId: ModalId, modal?: Partial<Modal<P>>) => void;
	opened: ModalId[];
	register: <P>(modal: Modal<P>) => void;
	registered: Record<ModalId, Modal<any>>;
	rootRef?: MutableRefObject<any>;
	unregister: (modalId: ModalId) => void;
};

export const ModalContext = createContext<ModalContextType>({
	registered: {},
	opened: [],
	open: () => {},
	close: () => {},
	register: () => {},
	unregister: () => {},
});

export const ModalProvider: FC<{ children: ReactNode }> = ({ children }) => {
	const [registered, setRegistered] = useState<Record<ModalId, Modal<any>>>({});
	const [opened, setOpened] = useState<ModalId[]>([]);

	const rootRef = useRef<View | null>(null);

	const register = function <P>(modal: Modal<P>) {
		setRegistered((modals) => ({
			...modals,
			[modal.id]: {
				...modals[modal.id],
				...modal,
			},
		}));
	};

	const unregister = (modalId: ModalId) => {
		setRegistered((modals) => omit([modalId], modals));
	};

	const open = function <P>(modalId: ModalId, modal: Modal<P> | undefined) {
		register({ ...modal, id: modalId });
		setOpened((opened) => [...opened, modalId]);
	};

	const close = (modalId: ModalId) => {
		setOpened((opened) => opened.filter((id) => id !== modalId));
	};

	useEffect(() => {
		DeviceEventEmitter.addListener('openModal', (e: CustomEventInit<Modal<any>>) => {
			if (e.detail) open(e.detail.id, e.detail);
		});
		DeviceEventEmitter.addListener('closeModal', (e: CustomEventInit<ModalId>) => {
			if (e.detail) close(e.detail);
		});
	}, []);

	const value = useMemo(
		() => ({
			registered: registered,
			opened,
			unregister,
			register,
			open,
			rootRef,
			close,
		}),
		[registered, opened],
	);

	return <ModalContext.Provider value={value}>{children}</ModalContext.Provider>;
};

export function useModal<P>(id?: Modal<P>['id'], options?: Omit<Modal<P>, 'id'>) {
	const { close, open, ...others } = useContext(ModalContext);
	return [
		(newOptions?: Omit<Modal<P>, 'id'>) =>
			open(id!, {
				...options,
				...newOptions,
			}),
		() => close(id!),
		{
			modals: others.registered,
			opened: others.opened,
			openModal: open,
			closeModal: close,
			registerModal: others.register,
			unregisterModal: others.unregister,
			rootRef: others.rootRef,
		},
	] as const;
}

export const openModal = (modal: Modal<any>) => {
	DeviceEventEmitter.emit('openModal', { detail: modal });
};

export const closeModal = (modalId: ModalId) => {
	DeviceEventEmitter.emit('closeModal', { detail: modalId });
};
