import React, { forwardRef, ReactNode } from 'react';
import { StyleProp, ViewStyle } from 'react-native';
import { Gesture, GestureDetector } from 'react-native-gesture-handler';
import Animated, {
	measure,
	runOnUI,
	useAnimatedStyle,
	useSharedValue,
	withTiming,
} from 'react-native-reanimated';

export type RippleEffectProps = {
	children: ReactNode;
	color?: string;
	disabled?: boolean;
	duration?: number;
	opacity?: number;
	style?: {
		container: StyleProp<ViewStyle>;
	};
};

export const RippleEffect = forwardRef(
	(
		{
			duration = 500,
			opacity = 0.8,
			color = 'white',
			children,
			style,
			disabled,
		}: RippleEffectProps,
		ref: any,
	) => {
		const centerX = useSharedValue(0);
		const centerY = useSharedValue(0);
		const scale = useSharedValue(0);
		const width = useSharedValue(0);
		const height = useSharedValue(0);

		const rippleOpacity = useSharedValue(opacity);

		const tapGestureEvent = Gesture.Tap()
			.enabled(!disabled)
			.runOnJS(true)
			.onBegin((tapEvent) => {
				scale.value = 0;
				centerX.value = tapEvent.x;
				centerY.value = tapEvent.y;
				rippleOpacity.value = opacity;

				runOnUI(() => {
					const layout = measure(ref);
					width.value = layout!.width;
					height.value = layout!.height;
				})();

				scale.value = withTiming(1, { duration: duration });
				rippleOpacity.value = withTiming(0, { duration: duration });
			})
			.onEnd(() => {
				rippleOpacity.value = withTiming(0, { duration: duration });
			});

		const animatedStyle = useAnimatedStyle(() => {
			const circleRadius = width.value > height.value ? width.value : height.value * 2;
			const translateX = centerX.value - circleRadius;
			const translateY = centerY.value - circleRadius;

			return {
				width: circleRadius * 2,
				height: circleRadius * 2,
				borderRadius: circleRadius,
				opacity: rippleOpacity.value,
				zIndex: -1,
				backgroundColor: color,
				position: 'absolute',
				top: 0,
				left: 0,
				transform: [
					{ translateX },
					{ translateY },
					{
						scale: scale.value,
					},
				],
			};
		}, [centerX, centerY, width, height, rippleOpacity, scale]);

		return (
			<GestureDetector gesture={tapGestureEvent}>
				<Animated.View
					style={[
						{
							overflow: 'hidden',
						},
						style?.container,
					]}>
					<Animated.View style={animatedStyle} />
					{children}
				</Animated.View>
			</GestureDetector>
		);
	},
);

RippleEffect.displayName = 'RippleEffect';
