import React, {
	createContext,
	FC,
	ReactNode,
	useContext,
	useEffect,
	useMemo,
	useState,
} from 'react';
import { omit } from 'ramda';

export type Toaster = {
	color?: 'primary' | 'secondary' | 'positive' | 'negative';
	id: string;
	message: string;
	timeout?: number;
	title?: string;
};

export type ToasterId = Toaster['id'];

type ToasterContextType = {
	open: (toaster: Toaster) => void;
	toasters: Record<ToasterId, Toaster>;
};

const DefaultTimeout = 6 * 1000;
const timeouts: { [key: ToasterId]: any } = [];

const ToasterContext = createContext<ToasterContextType>({
	toasters: {},
	open: () => {},
});

export const ToasterProvider: FC<{ children: ReactNode }> = ({ children }) => {
	const [toasters, setToasters] = useState<Record<ToasterId, Toaster>>({});

	const autoHide = (toaster: Toaster) => {
		const { id } = toaster;
		setToasters({ ...toasters, [id]: toaster });
		timeouts[id] = setTimeout(() => {
			setToasters(omit([id], toasters));
		}, toaster.timeout ?? DefaultTimeout);
	};

	useEffect(
		() => () => {
			Object.keys(timeouts).forEach((timeoutId) => {
				clearTimeout(timeouts[timeoutId]);
			});
		},
		[],
	);

	useEffect(() => {
		window.addEventListener('showToaster', (e: CustomEventInit<Toaster>) => {
			if (e.detail) autoHide(e.detail);
		});
	}, []);

	const value = useMemo(
		() => ({
			toasters,
			open: autoHide,
		}),
		[toasters],
	);

	return <ToasterContext.Provider value={value}>{children}</ToasterContext.Provider>;
};

export const showToaster = (toaster: Toaster) => {
	window.dispatchEvent(new CustomEvent<Toaster>('showToaster', { detail: toaster }));
};

export const useToaster = () => {
	const { open, toasters } = useContext(ToasterContext);

	return {
		toasters,
		showToaster: open,
	};
};
