import React, {
	createContext,
	Dispatch,
	FC,
	ReactNode,
	SetStateAction,
	useContext,
	useState,
} from 'react';
import { mergeDeepRight } from 'ramda';
import { Theme, ThemeProvider as EmotionThemeProvider } from '@emotion/react';
import invariant from 'tiny-invariant';

type ThemeContext = {
	setTheme: Dispatch<SetStateAction<Theme>>;
	theme: Theme;
	updateTheme: (theme: Partial<Theme>) => void;
};

const ThemeContext = createContext<ThemeContext>({
	theme: {} as Theme,
	setTheme: () => {},
	updateTheme: () => {},
});

export type ThemeProviderProps = {
	children: ReactNode;
	theme: Theme;
};

export const ThemeProvider: FC<ThemeProviderProps> = ({
	theme: initialTheme,
	children,
}) => {
	const [theme, setTheme] = useState<Theme>(initialTheme);

	const updateTheme = (modifiedTheme: Partial<Theme>) => {
		if (!theme) return;
		const value = mergeDeepRight<Theme, Partial<Theme>>(theme ?? {}, modifiedTheme);
		setTheme(value as Theme);
	};

	return (
		<ThemeContext.Provider
			value={{
				theme,
				setTheme,
				updateTheme,
			}}>
			<EmotionThemeProvider theme={theme ?? {}}>{children}</EmotionThemeProvider>
		</ThemeContext.Provider>
	);
};

export const useTheme = () => {
	const context = useContext(ThemeContext);
	invariant(context, 'useTheme must be used within a ThemeProvider');
	return context;
};
