import Values from 'values.js';

type RGB = [number, number, number];
type HSL = [number, number, number];

// Convert an RGB color to HSL
export function RGBToHSL([r1, g1, b1]: RGB): HSL {
	r1 /= 255;
	g1 /= 255;
	b1 /= 255;

	const max = Math.max(r1, g1, b1);
	const min = Math.min(r1, g1, b1);

	let h: number = 0;
	let s: number = 0;
	let l: number = (max + min) / 2;

	if (max !== min) {
		s = l < 0.5 ? (max - min) / (max + min) : (max - min) / (2 - max - min);
		h =
			max === r1
				? (g1 - b1) / (max - min)
				: max === g1
					? 2 + (b1 - r1) / (max - min)
					: 4 + (r1 - g1) / (max - min);
	}

	h *= 60;
	if (h < 0) h += 360;
	s *= 100;
	l *= 100;

	return [h, s, l];
}

// Convert an HSL color to RGB
export function HSLToRGB([h, s, l]: HSL): RGB {
	s /= 100;
	l /= 100;

	const c: number = (1 - Math.abs(2 * l - 1)) * s;
	const x: number = c * (1 - Math.abs(((h / 60) % 2) - 1));
	const m: number = l - c / 2;

	let r: number = 0;
	let g: number = 0;
	let b: number = 0;
	if (0 <= h && h < 60) {
		r = c;
		g = x;
		b = 0;
	} else if (60 <= h && h < 120) {
		r = x;
		g = c;
		b = 0;
	} else if (120 <= h && h < 180) {
		r = 0;
		g = c;
		b = x;
	} else if (180 <= h && h < 240) {
		r = 0;
		g = x;
		b = c;
	} else if (240 <= h && h < 300) {
		r = x;
		g = 0;
		b = c;
	} else if (300 <= h && h < 360) {
		r = c;
		g = 0;
		b = x;
	}

	r = Math.round((r + m) * 255);
	g = Math.round((g + m) * 255);
	b = Math.round((b + m) * 255);

	return [r, g, b];
}

// Calculate the contrast color
export function getContrastColor(rgb: RGB): string {
	const hsl = RGBToHSL(rgb);
	hsl[0] = (hsl[0] + 180) % 360;
	const contrastRGB = HSLToRGB(hsl);
	return contrastRGB.map((v) => v.toString(16).padStart(2, '0')).join('');
}

export const isDark = (color: string) => {
	return new Values(color).getBrightness() <= 55;
};
