import './styles/global.css';

import React, { useEffect } from 'react';
import { GestureHandlerRootView } from 'react-native-gesture-handler';
import { INITIAL_VIEWPORTS } from '@storybook/addon-viewport';
import { mergeDeepRight } from 'ramda';
import { ThemeProvider, useTheme } from '@procyon/styles/Theme';

import {
	Controls,
	Description,
	Primary,
	Stories,
	Subtitle,
	Title,
} from '@storybook/blocks';

import { TolgeeChain, TolgeeProvider, useTolgee } from '@procyon/localization/tolgee';
import { buildColorTheme } from '@procyon/styles/theme/color';
import { baseTheme } from '@procyon/styles/theme/base';
import { MediaQueryProvider } from '@procyon/styles/useMediaQuery';
import { ModalProvider } from '@procyon/components/Modal/context';
import { Modals } from '@procyon/components/Modal';
import { Theme } from '@emotion/react';
import { getContrastColor } from '@procyon/styles/utils/color';
import Values from 'values.js';

export const tolgee = TolgeeChain.init({
	apiKey: process.env.TOLGEE_API_KEY,
	apiUrl: process.env.TOLGEE_API_URL,
	language: 'en',
	ns: ['procyon', 'button', 'calendar'],
});

const LIGHT_COLORS = {
	primary: '#674ddb',
	highlight: `#${getContrastColor(new Values('#674ddb').rgb)}`,
	secondary: '#b8b8b8',
	positive: '#66bb6a',
	negative: '#f32c1e',
	base: '#fff',
	text: '#000',
};

export const themes = {
	light: mergeDeepRight(buildColorTheme(LIGHT_COLORS), baseTheme),
};

export const globalTypes = {
	locale: {
		name: 'Locale',
		defaultValue: 'en',
		toolbar: {
			icon: 'globe',
			dynamicTitle: true,
			items: [
				{ value: 'en', right: '🇺🇸', title: 'English' },
				{ value: 'cs', right: '🇨🇿', title: 'Čeština' },
			],
		},
	},
	theme: {
		name: 'Theme',
		defaultValue: 'light',
		toolbar: {
			icon: 'facehappy',
			dynamicTitle: true,
			items: [
				{ value: 'light', right: '🟣', title: 'Light (purple)' },
				{ value: 'light2', right: '🟠', title: 'Light (orange)' },
				// { value: 'dark', right: '🌙', title: 'Dark' },
			],
		},
	},
};

export const decorators = [
	(Children, context) => {
		const sbLocale = context.globals;
		const tolgee = useTolgee();

		useEffect(() => {
			tolgee.changeLanguage(sbLocale.locale);
		}, [sbLocale.locale]);
		return <Children />;
	},
	(Children, context) => {
		const sbTheme = context.globals;
		const { updateTheme } = useTheme();

		useEffect(() => {
			updateTheme(
				buildColorTheme({
					...LIGHT_COLORS,
					...(sbTheme.theme === 'light2' && {
						primary: '#db6f4d',
						highlight: `#${getContrastColor(new Values('#db6f4d').rgb)}`,
					}),
				}),
			);
		}, [sbTheme.theme]);

		return <Children />;
	},
	(Children) => {
		return (
			<GestureHandlerRootView>
				<ThemeProvider theme={themes.light as Theme}>
					<TolgeeProvider tolgee={tolgee}>
						<ModalProvider>
							<MediaQueryProvider>
								<Children />
								<Modals />
							</MediaQueryProvider>
						</ModalProvider>
					</TolgeeProvider>
				</ThemeProvider>
			</GestureHandlerRootView>
		);
	},
];

export const parameters = {
	viewport: {
		viewports: INITIAL_VIEWPORTS,
	},
	options: {
		storySort: {
			method: 'alphabetical-by-kind',
			order: [
				'Introduction',
				'Auth',
				['Introduction'],
				'components',
				['Inputs', 'Layout', 'Display', 'Utils', '*'],
				'Form',
				['Introduction', 'Implementation', '*'],
				'Localization',
				['Introduction', 'Example'],
				'Styles',
				['Introduction', '*'],
			],
		},
	},
	docs: {
		page: () => (
			<>
				<Title />
				<Subtitle />
				<Description />
				<Primary />
				<Controls />
				<Stories />
			</>
		),
	},
};
