import React, { FC, ReactNode, useState } from 'react';
import styled from '@emotion/native';
import { Theme } from '@emotion/react';
import { View } from 'react-native';

import { Modals } from '@procyon/components/Modal/Modals';
import { Toasters } from '@procyon/components/Toaster';
import { ToastersProps } from '@procyon/components/Toaster/Toasters';
import { MenuIcon } from '@procyon/icons/Menu';
import { useMediaQuery } from '@procyon/styles/useMediaQuery';

export type AppLayoutProps = {
	Footer?: FC;
	Logo?: ReactNode;
	SideNav?: FC[];
	TopNav?: FC[];
	UserNav?: FC[];
	children: ReactNode;
	toasterPosition?: ToastersProps['position'];
};

export const AppLayout: React.FC<AppLayoutProps> = ({
	children,
	Footer = View,
	TopNav,
	Logo,
	UserNav,
	SideNav,
	toasterPosition = 'topRight',
}) => {
	const enabledSideNav = !!SideNav;
	const enabledTopNav = !!Logo || !!TopNav || !!UserNav;

	const mq = useMediaQuery();

	const [sideCollapsed, setSideCollapsed] = useState(false);
	const [topCollapsed, setTopCollapsed] = useState(true);

	const toggleSide = () => {
		setSideCollapsed(!sideCollapsed);
	};

	const toggleTop = () => {
		setTopCollapsed(!topCollapsed);
	};

	return (
		<AppLayoutContainer>
			{enabledTopNav && (
				<AppLayoutTopNav mq={mq} onPress={toggleTop}>
					<AppLayoutTopLeftSide mq={mq} placeholder={!Logo}>
						<AppLayoutLogo>{Logo ?? <View />}</AppLayoutLogo>
						<AppLayoutMenuToggle
							mq={mq}
							style={{
								marginBottom: 2,
							}}
							strokeWidth={1}
						/>
					</AppLayoutTopLeftSide>
					<AppLayoutNavSection
						mq={mq}
						visible={!topCollapsed}
						direction="row"
						justifyContent="flex-start">
						{TopNav?.map((Nav: any, i) => <Nav key={i} />)}
					</AppLayoutNavSection>
					<AppLayoutNavSection
						mq={mq}
						visible={!topCollapsed}
						direction="row"
						justifyContent="flex-end">
						{UserNav?.map((Nav: any, i) => <Nav key={i} />)}
					</AppLayoutNavSection>
				</AppLayoutTopNav>
			)}
			<AppLayoutContent mq={mq}>
				{enabledSideNav && (
					<AppLayoutSideNav mq={mq} onPress={toggleSide}>
						<AppLayoutMenuToggle mq={mq} style={{ minWidth: 30 }} strokeWidth={1} />
						<AppLayoutNavSection
							mq={mq}
							visible={sideCollapsed}
							direction="column"
							justifyContent="center">
							{SideNav?.map((Nav: any, i) => <Nav side key={i} />)}
						</AppLayoutNavSection>
					</AppLayoutSideNav>
				)}
				<AppLayoutPage>{children}</AppLayoutPage>
			</AppLayoutContent>
			<Footer />
			<Modals />
			<Toasters position={toasterPosition} />
		</AppLayoutContainer>
	);
};

const AppLayoutContainer = styled(View)(({ theme }) => {
	return {
		backgroundColor: theme.palette.background.main,
	};
});

const AppLayoutNavSection = styled(View)<{
	direction: 'row' | 'column';
	justifyContent: 'flex-start' | 'center' | 'flex-end';
	mq: keyof Theme['breakpoints'];
	visible: boolean;
}>(({ theme, direction, visible, justifyContent, mq }) => {
	return {
		display: 'flex',
		flexDirection: direction,
		columnGap: 4,
		rowGap: 4,

		...(mq === 'xs' && {
			marginTop: theme.spacing.sm,
			flexWrap: 'wrap',
			flex: 1,
			justifyContent: justifyContent,
			...(visible
				? {}
				: {
						display: 'none',
					}),
		}),
	};
});

const AppLayoutTopNav = styled.Pressable<{ mq: keyof Theme['breakpoints'] }>(
	({ theme, mq }) => {
		return {
			backgroundColor: theme.palette.primary.light,
			display: 'flex',
			flexDirection: 'row',
			justifyContent: 'space-between',
			columnGap: 4,
			alignItems: 'center',
			padding: theme.spacing.sm,
			cursor: 'auto',

			...(mq === 'xs' && {
				flexDirection: 'column',
				alignItems: 'stretch',
			}),
		};
	},
);

const AppLayoutTopLeftSide = styled(View)<{
	mq: keyof Theme['breakpoints'];
	placeholder: boolean;
}>(({ placeholder, mq }) => {
	return {
		...(placeholder && {
			display: 'none',
		}),

		...(mq === 'xs' && {
			display: 'flex',
			flexDirection: 'row',
			justifyContent: 'space-between',
			alignItems: 'center',
		}),
	};
});

const AppLayoutLogo = styled.Text(({ theme }) => {
	return {
		color: theme.palette.primary.dark,
		fontFamily: theme.fontFamily,
		fontSize: theme.fontSize.xl,
	};
});

const AppLayoutMenuToggle = styled(MenuIcon)<{
	mq: keyof Theme['breakpoints'];
}>(({ mq }) => {
	return {
		display: 'none',
		...(mq === 'xs' && {
			display: 'flex',
		}),
	};
});

const AppLayoutContent = styled(View)<{
	mq: keyof Theme['breakpoints'];
}>(({ theme, mq }) => {
	return {
		display: 'flex',
		flexDirection: 'row',
		gap: 4,
		marginTop: theme.spacing.md,

		...(mq === 'xs' && {
			flexDirection: 'column',
			marginTop: theme.spacing.sm,
		}),
	};
});

const AppLayoutSideNav = styled.Pressable<{
	mq: keyof Theme['breakpoints'];
}>(({ theme, mq }) => {
	return {
		width: 150,
		borderRadius: theme.rounded.small,
		backgroundColor: theme.palette.secondary.light,
		display: 'flex',
		flexDirection: 'column',
		justifyContent: 'space-between',
		rowGap: 4,
		padding: theme.spacing.sm,
		marginRight: 2,

		...(mq === 'xs' && {
			width: 'auto',
			flexDirection: 'row',
			flexWrap: 'wrap',
			marginHorizontal: theme.spacing.md,
		}),
	};
});

const AppLayoutPage = styled(View)(({ theme }) => {
	return {
		padding: theme.spacing.md,
		flex: 1,
	};
});
