import React, { ReactElement, ReactNode, useMemo, useState } from 'react';
import styled, { css } from '@emotion/native';
import { Platform, StyleProp, TextStyle, View, ViewStyle } from 'react-native';
import { useAnimatedRef } from 'react-native-reanimated';
import Values from 'values.js';

import { RippleEffect } from '@procyon/components/RippleEffect';
import { Stack } from '@procyon/components/Stack';
import { useTheme } from '@procyon/styles/Theme';
import { useMediaQuery } from '@procyon/styles/useMediaQuery';
import { isDark } from '@procyon/styles/utils/color';

export type ButtonProps = {
	LeftIcon?: ReactElement;
	RightIcon?: ReactElement;
	children: ReactNode;
	color?: 'primary' | 'secondary' | 'positive' | 'negative';
	disabled?: boolean;
	fullWidth?: boolean;
	onClick?: (e?: any) => void | Promise<void>;
	ripple?: {
		color?: string;
	};
	size?: 'sm' | 'md' | 'lg' | 'xl';
	style?: {
		container?: StyleProp<ViewStyle>;
		text?: StyleProp<TextStyle>;
	};
	type?: 'button' | 'submit';
	variant?: 'contained' | 'outlined' | 'text';
};

export const Button: React.FC<ButtonProps> = (props) => {
	const {
		disabled,
		onClick,
		color = 'primary',
		size = 'md',
		children,
		variant = 'contained',
		fullWidth,
		style,
		LeftIcon,
		RightIcon,
		ripple,
	} = props;

	const rippleAnimateRef = useAnimatedRef<View>();

	const [hover, setHover] = useState(false);

	const mq = useMediaQuery();
	const { theme } = useTheme();

	const rippleColor = useMemo(() => {
		if (ripple?.color) return ripple.color;
		const val = theme.palette[color!].main;
		if (isDark(val)) {
			return new Values(val).tint(30).rgbString();
		} else {
			return new Values(val).shade(30).rgbString();
		}
	}, [color]);

	const buttonColor = useMemo(() => {
		if (variant === 'contained') {
			return isDark(theme.palette[color!].main)
				? theme.palette[color!].light
				: theme.palette[color!].dark;
		}
		if (variant === 'outlined' || variant === 'text') {
			return theme.palette[color!].dark;
		}
		if (disabled) {
			return theme.palette.disabled;
		}
	}, [color, variant, disabled]);

	const LeftIconComponent = LeftIcon ?? null;
	const RightIconComponent = RightIcon ?? null;

	return (
		<ButtonContainer
			variant={variant}
			aria-label="button"
			color={color}
			onPress={(...args) => onClick?.(...args)}
			hover={hover}
			fullWidth={fullWidth || (fullWidth === undefined && mq === 'xs')}
			onHoverIn={() => setHover(true)}
			onHoverOut={() => setHover(false)}
			style={[style?.container]}
			disabled={disabled}
			collapsable={false}
			ref={rippleAnimateRef}>
			<RippleEffect
				ref={rippleAnimateRef}
				disabled={disabled}
				color={rippleColor}
				style={{
					container: [
						css({
							paddingHorizontal: theme.spacing.lg,
							borderRadius: theme.rounded.small,
							alignItems: 'center',
						}),
						style?.container,
					],
				}}>
				<Stack direction="row" gap={1} alignItems="center">
					{LeftIconComponent && (
						<LeftIconComponent.type
							{...LeftIconComponent.props}
							color={buttonColor}
							size={theme.iconSize[size!]}
						/>
					)}
					<ButtonText
						variant={variant}
						size={size}
						style={[{ color: buttonColor }, style?.text]}
						disabled={disabled}>
						{children}
					</ButtonText>
					{RightIconComponent && (
						<RightIconComponent.type {...RightIconComponent.props} color={buttonColor} />
					)}
				</Stack>
			</RippleEffect>
		</ButtonContainer>
	);
};

const ButtonContainer = styled.Pressable<{
	color: ButtonProps['color'];
	fullWidth: ButtonProps['fullWidth'];
	hover: boolean;
	variant: ButtonProps['variant'];
}>(
	({ theme }) => ({
		borderRadius: theme.rounded.small,
		backgroundColor: 'transparent',
		justifyContent: 'center',
	}),
	({ theme, variant, color, hover, disabled, fullWidth }) => ({
		...(!fullWidth && {
			...(Platform.OS === 'web'
				? {
						width: 'fit-content' as any,
					}
				: {
						display: 'flex',
						alignSelf: 'flex-start',
					}),
		}),

		...(variant === 'contained'
			? {
					backgroundColor: theme.palette[color!].main,
					...(hover && {
						backgroundColor: new Values(theme.palette[color!].main).shade(5).rgbString(),
					}),
					...(color === 'secondary' && {
						backgroundColor: theme.palette.secondary.light,
						...(hover && {
							backgroundColor: new Values(theme.palette.secondary.light)
								.shade(5)
								.rgbString(),
						}),
					}),
					...(disabled && {
						backgroundColor: new Values(theme.palette.background.main)
							.tint(60)
							.rgbString(),
					}),
				}
			: {}),
		...(variant === 'outlined'
			? {
					borderWidth: 2,
					borderColor: theme.palette[color!].main,
					...(hover && {
						backgroundColor: new Values(theme.palette[color!].main).tint(90).rgbString(),
					}),
					...(disabled && {
						borderColor: new Values(theme.palette.background.main).tint(60).rgbString(),
					}),
				}
			: {}),
		...(variant === 'text'
			? {
					...(hover && {
						backgroundColor: new Values(theme.palette[color!].main).tint(90).rgbString(),
					}),
				}
			: {}),
	}),
);

const ButtonText = styled.Text<{
	size: ButtonProps['size'];
	variant: ButtonProps['variant'];
}>(({ theme, size }) => ({
	fontFamily: theme.fontFamily,
	padding: theme.spacing[size!],
	fontSize: theme.fontSize[size!],
	fontWeight: 'normal',
	userSelect: 'none',
	textTransform: 'uppercase',
	display: 'flex',
	justifyContent: 'center',
	alignItems: 'center',
	gap: 4,
}));
