import React, {
	Children,
	forwardRef,
	isValidElement,
	MutableRefObject,
	ReactNode,
} from 'react';
import { css } from '@emotion/native';
import { Platform, StyleProp, View, ViewStyle } from 'react-native';

import { ButtonProps } from '@procyon/components/Button';
import { useTheme } from '@procyon/styles/Theme';
import { notNil } from '@procyon/utils/nil';

export type ButtonGroupProps = {
	children: ReactNode[];
	style?: StyleProp<ViewStyle>;
} & Partial<ButtonProps>;

export const ButtonGroup = forwardRef(
	(
		{
			children,
			style,
			color = 'primary',
			variant = 'contained',
			...buttonProps
		}: ButtonGroupProps,
		ref: MutableRefObject<View>,
	) => {
		const { theme } = useTheme();
		const filteredChildren = children.filter(notNil);

		return (
			<View
				ref={ref}
				aria-label="button-group"
				style={[
					css({
						flexDirection: 'row',
						...(Platform.OS === 'web'
							? {
									width: 'fit-content' as any,
								}
							: {
									display: 'flex',
									alignSelf: 'flex-start',
								}),
					}),
					style,
				]}>
				{Children.map(filteredChildren, (child, index) => {
					const style: ButtonProps['style'] = {};

					if (filteredChildren.length <= 1) {
						style.container = {};
					} else {
						// first button
						if (index === 0) {
							style.container = {
								borderTopRightRadius: 0,
								borderBottomRightRadius: 0,
								borderRightWidth: 1,
								borderRightColor: theme.palette.primary.dark,
							};
						}
						// last button
						else if (index === children.length - 1) {
							style.container = {
								borderTopLeftRadius: 0,
								borderBottomLeftRadius: 0,
								// do not set border if there are only two buttons (like dropdown)
								...(filteredChildren.length > 2 && {
									borderLeftWidth: 1,
									borderLeftColor: theme.palette.primary.dark,
								}),
							};
						} else {
							style.container = {
								borderRadius: 0,
							};
						}
					}

					if (isValidElement(child)) {
						return {
							...child,
							props: {
								color,
								variant,
								...buttonProps,
								...child.props,
								style: {
									...child.props.style,
									container: {
										...style.container,
										...(child.props?.style?.container ?? {}),
									},
								},
							} as ButtonProps,
						};
					}
					return child;
				})}
			</View>
		);
	},
);

ButtonGroup.displayName = 'ButtonGroup';
