import React, { FC, Fragment, useEffect, useMemo, useState } from 'react';
import { range } from 'ramda';
import styled from '@emotion/native';
import { add, endOfMonth, getWeeksInMonth, startOfMonth, sub } from 'date-fns';

import { Grid } from '@procyon/components/Grid';
import { Stack } from '@procyon/components/Stack';
import { useTheme } from '@procyon/styles/Theme';
import { notNil } from '@procyon/utils/nil';

import { CalendarWeek } from './CalendarWeek';
import { Controls, ControlsProps } from './Controls';
import { DateSlot, DateSlotProps } from './DateSlot';
import { getWeekRange } from './utils';
import { WeekDays, WeekDaysProps } from './WeekDays';

export type CalendarProps = {
	ControlsView?: FC<ControlsProps>;
	DateSlotView?: FC<DateSlotProps>;
	DaysOfWeekView?: FC<WeekDaysProps>;
	TimeColumn?: FC;
	layout?: 'day' | 'month' | 'week';
	longDayNames?: boolean;
	onClick?: (date: Date) => void;
	onRangeChange?: (dateFrom: Date, dateEnd: Date) => void;
	selectWeek?: boolean;
	selected?: Date;
	style?: {
		control?: ControlsProps['style'];
		daySlot?: DateSlotProps['style'];
		weekDays?: WeekDaysProps['style'];
	};
	title?: string;
	type?: 'days' | 'months';
	value?: Date;
	weekOfMonth?: number;
};

export function Calendar({
	value,
	onClick,
	onRangeChange,
	layout = 'month',
	type = 'days',
	weekOfMonth,
	ControlsView = Controls,
	DaysOfWeekView = WeekDays,
	title,
	DateSlotView = DateSlot,
	TimeColumn = Fragment,
	selectWeek,
	selected,
	style,
	longDayNames,
}: CalendarProps): ReturnType<FC> {
	const [workDate, setWorkDate] = useState<Date>(value ?? new Date());

	const { theme } = useTheme();

	const handleChangeRange = () => {
		switch (layout) {
			case 'month':
				onRangeChange?.(startOfMonth(workDate), endOfMonth(workDate));
				return;
			case 'week':
				onRangeChange?.(getWeekRange(workDate)[0], getWeekRange(workDate)[1]);
				return;
			case 'day':
				onRangeChange?.(workDate, workDate);
		}
	};

	const handleClickPrev = () => {
		switch (layout) {
			case 'month':
				if (type === 'months') {
					setWorkDate(sub(workDate, { years: 1 }));
					return;
				}
				setWorkDate(sub(workDate, { months: 1 }));
				return;
			case 'week':
				setWorkDate(sub(workDate, { weeks: 1 }));
				return;
			case 'day':
				setWorkDate(sub(workDate, { days: 1 }));
		}
	};

	const handleClickNext = () => {
		switch (layout) {
			case 'month':
				if (type === 'months') {
					setWorkDate(add(workDate, { years: 1 }));
					return;
				}
				setWorkDate(add(workDate, { months: 1 }));
				return;
			case 'week':
				setWorkDate(add(workDate, { weeks: 1 }));
				return;
			case 'day':
				setWorkDate(add(workDate, { days: 1 }));
		}
	};

	const handleClickSetToday = () => {
		setWorkDate(new Date());
	};

	const handleClick = (date: Date) => {
		onClick?.(date);
	};

	useEffect(() => {
		if (value && value !== workDate) {
			setWorkDate(value);
		}
	}, [value]);

	useEffect(() => {
		handleChangeRange();
	}, [workDate]);

	const DaysLayout = useMemo(() => {
		const end = layout === 'month' ? getWeeksInMonth(workDate, { weekStartsOn: 1 }) : 1;
		return (
			<Stack direction="column" width="100%">
				<DaysOfWeekView layout={layout} workDate={workDate} longNames={longDayNames} />
				{range(0, end).map((i: number) => (
					<CalendarWeek
						key={i}
						selected={selected}
						weekOfMonth={i}
						layout={layout}
						DateSlotView={DateSlotView}
						onClick={handleClick}
						workDate={new Date(workDate)}
						value={value}
						type={type}
						selectWeek={selectWeek}
					/>
				))}
			</Stack>
		);
	}, [
		layout,
		selected,
		DateSlotView,
		WeekDays,
		workDate,
		value,
		type,
		longDayNames,
		selectWeek,
	]);

	const MonthsLayout = useMemo(() => {
		return (
			<Grid cols={{ xs: 3 }}>
				{range(0, 3).map((rowIndex) => {
					return range(1, 5).map((cellIndex) => {
						const month = cellIndex + 4 * rowIndex;
						return (
							<DateSlotView
								key={cellIndex}
								date={new Date(workDate.getFullYear(), month - 1, 1)}
								workDate={workDate}
								layout={layout}
								isSelected={
									selected?.getMonth() === month - 1 &&
									workDate.getFullYear() === selected?.getFullYear()
								}
								isToday={
									new Date().getMonth() === month - 1 &&
									workDate.getFullYear() === new Date().getFullYear()
								}
								type={type}
								weekOfMonth={weekOfMonth}
								onClick={handleClick}>
								{month}
							</DateSlotView>
						);
					});
				})}
			</Grid>
		);
	}, [workDate, type, layout, onClick]);

	return (
		<Stack direction="column" gap={theme.spacing.xl}>
			{notNil(title) && <CalendarTitle>{title}</CalendarTitle>}
			<ControlsView
				layout={layout}
				onNextClick={handleClickNext}
				onPrevClick={handleClickPrev}
				onTodayClick={handleClickSetToday}
				type={type}
				workDate={workDate}
				style={style?.control}
			/>
			<Stack direction="row" justifyContent="center">
				<TimeColumn />
				{type === 'days' && DaysLayout}
				{type === 'months' && MonthsLayout}
			</Stack>
		</Stack>
	);
}

const CalendarTitle = styled.Text(({ theme }) => {
	return {
		fontSize: theme.fontSize.lg,
		fontFamily: theme.fontFamily,
		marginBottom: theme.spacing.md,
	};
});
