import React, { FC, ReactNode, useMemo } from 'react';
import styled from '@emotion/native';
import { isSameMonth } from 'date-fns';
import { Pressable, StyleProp, TextStyle, View, ViewStyle } from 'react-native';
import { useAnimatedRef } from 'react-native-reanimated';

import { CalendarProps } from '@procyon/components/Calendar/index';
import { RippleEffect } from '@procyon/components/RippleEffect';
import { useHover } from '@procyon/hooks/useHover';
import { useTheme } from '@procyon/styles/Theme';
import { debounceAfter } from '@procyon/utils/debounce';

export type DateSlotProps = {
	children?: ReactNode;
	date: Date;
	isSelected?: boolean;
	isToday?: boolean;
	layout?: CalendarProps['layout'];
	onClick?: (date: Date) => void;
	style?: {
		container?: StyleProp<ViewStyle>;
		date?: StyleProp<TextStyle>;
	};
	type?: CalendarProps['type'];
	weekOfMonth?: number;
	workDate: Date;
};

export const DateSlot: FC<DateSlotProps> = ({
	children,
	workDate,
	date,
	isSelected,
	isToday,
	onClick,
	type,
	style,
}) => {
	const isOtherMonth = !isSameMonth(workDate, date);

	const rippleAnimateRef = useAnimatedRef<View>();
	const { hovered, onHoverIn, onHoverOut } = useHover();
	const { theme } = useTheme();

	const rippleColor = useMemo(() => {
		if (isToday || isSelected) {
			return theme.palette.primary.light;
		} else {
			return theme.palette.highlight.main;
		}
	}, []);

	return (
		<Pressable
			onPress={debounceAfter(() => onClick?.(date), 200)}
			onHoverIn={onHoverIn}
			onHoverOut={onHoverOut}>
			<CalendarDateContainer
				style={style?.container}
				type={type}
				isSelected={isSelected}
				isOtherMonth={isOtherMonth}
				hovered={hovered}
				ref={rippleAnimateRef}>
				<RippleEffect
					color={rippleColor}
					style={{
						container: {
							alignItems: 'center',
							justifyContent: 'center',
							borderRadius: theme.rounded.small,
						},
					}}
					ref={rippleAnimateRef}>
					<CalendarDateText
						style={style?.container}
						type={type}
						isToday={isToday}
						isSelected={isSelected}
						isOtherMonth={isOtherMonth}>
						{children}
					</CalendarDateText>
				</RippleEffect>
			</CalendarDateContainer>
		</Pressable>
	);
};

const CalendarDateContainer = styled(View)<{
	hovered: boolean;
	isOtherMonth: boolean;
	isSelected: DateSlotProps['isSelected'];
	type: DateSlotProps['type'];
}>(({ theme, isSelected, hovered }) => {
	return {
		borderRadius: theme.rounded.small,
		margin: 1,
		padding: 2,

		...(hovered && {
			borderWidth: 1,
			padding: 1,
			backgroundColor: theme.palette.highlight.light,
			borderColor: theme.palette.highlight.dark,
		}),

		...(isSelected && {
			borderWidth: 1,
			padding: 1,
			backgroundColor: theme.palette.primary.light,
			borderColor: theme.palette.primary.dark,
		}),
	};
});

const CalendarDateText = styled.Text<{
	isOtherMonth: boolean;
	isSelected: DateSlotProps['isSelected'];
	isToday: DateSlotProps['isToday'];
	type: DateSlotProps['type'];
}>(({ theme, type, isSelected, isToday, isOtherMonth }) => {
	return {
		fontFamily: theme.fontFamily,
		fontSize: theme.fontSize.md,
		color: theme.palette.text.main,
		padding: theme.spacing.md,
		width: 36,
		textAlign: 'center',

		...(isOtherMonth &&
			type !== 'months' && {
				color: theme.palette.text.light,
			}),

		...(isToday && {
			fontWeight: 'bold',
		}),

		...(isSelected && {
			color: theme.palette.primary.dark,
		}),
	};
});
