import React, { useEffect } from 'react';
import styled from '@emotion/native';
import { NativeTouchEvent, StyleProp, TextStyle, View, ViewStyle } from 'react-native';
import { NativeSyntheticEvent } from 'react-native/Libraries/Types/CoreEventTypes';
import Animated, {
	useAnimatedStyle,
	useSharedValue,
	withTiming,
} from 'react-native-reanimated';

import { FieldMessage } from '@procyon/components/FieldMessage';
import { Label } from '@procyon/components/Label';
import { useMediaQuery } from '@procyon/styles/useMediaQuery';
import { InputElementType } from '@procyon/types/form';

export type CheckboxProps<V = boolean> = Omit<
	{
		checked?: boolean;
		description?: string;
		labelPosition?: 'left' | 'right';
		style?: {
			container?: StyleProp<ViewStyle>;
			description?: StyleProp<TextStyle>;
			input?: StyleProp<ViewStyle>;
			slider?: StyleProp<ViewStyle>;
			switch?: StyleProp<ViewStyle>;
		};
		switch?: boolean;
	} & InputElementType<V>,
	'value' | 'onBlur' | 'onFocus' | 'onClick'
>;

const SLIDER_X = {
	sm: { left: 0, right: 8 },
	md: { left: 2, right: 21 },
	lg: { left: 2, right: 30 },
	xl: { left: 4, right: 40 },
};

export const Checkbox: React.FC<CheckboxProps> = ({
	label,
	description,
	color = 'primary',
	disabled,
	size = 'md',
	labelSize,
	labelPosition = 'right',
	style,
	message,
	checked,
	onChange,
	fullWidth,
}) => {
	const mq = useMediaQuery();
	const sliderLeft = useSharedValue(checked ? SLIDER_X[size].right : SLIDER_X[size].left);
	const animatedStyles = useAnimatedStyle(() => {
		return {
			marginLeft: sliderLeft.value,
		};
	});

	useEffect(() => {
		sliderLeft.value = withTiming(checked ? SLIDER_X[size].right : SLIDER_X[size].left);
	}, [checked]);

	const handleClick = (e: NativeSyntheticEvent<NativeTouchEvent>) => {
		onChange?.(e, !checked);
	};

	return (
		<View
			style={[
				{
					width: fullWidth || (fullWidth === undefined && mq === 'xs') ? '100%' : 'auto',
				},
				style?.container,
			]}
			aria-label="checkbox">
			{label && (
				<Label size={labelSize} disabled={disabled}>
					{label}
				</Label>
			)}
			<CheckboxInput
				style={[style?.input]}
				onPress={handleClick}
				disabled={disabled}
				labelPosition={labelPosition}
				size={size}>
				<CheckboxSwitch
					color={color}
					checked={checked}
					disabled={disabled}
					size={size}
					style={[style?.switch]}>
					<CheckboxSlider
						style={[style?.slider, animatedStyles]}
						checked={checked}
						color={color}
						disabled={disabled}
						size={size}
					/>
				</CheckboxSwitch>
				<CheckboxDescription style={[style?.description]} disabled={disabled} size={size}>
					{description}
				</CheckboxDescription>
			</CheckboxInput>
			<FieldMessage color={color} size={size} message={message} />
		</View>
	);
};

const CheckboxInput = styled.Pressable<{
	labelPosition: CheckboxProps['labelPosition'];
	size: CheckboxProps['size'];
}>(
	() => {
		return {
			display: 'flex',
			alignItems: 'center',
			gap: 12,
			width: 'fit-content' as any,
		};
	},
	({ theme, size }) => {
		return {
			paddingTop: theme.spacing[size!] - 1,
			paddingBottom: theme.spacing[size!] + 4,
		};
	},
	({ labelPosition }) => {
		if (labelPosition === 'left')
			return {
				flexDirection: 'row-reverse',
			};
		else return { flexDirection: 'row' };
	},
);

const CheckboxSwitch = styled(View)<{
	checked: CheckboxProps['checked'];
	color: CheckboxProps['color'];
	disabled: CheckboxProps['disabled'];
	size: CheckboxProps['size'];
}>(
	() => {
		return {
			justifyContent: 'center',
			borderRadius: 50,
			borderWidth: 1,
		};
	},
	({ color, theme, checked }) => {
		return {
			borderColor: theme.palette[color!].dark,
			backgroundColor: checked
				? theme.palette[color!].light
				: theme.palette.background.main,
		};
	},
	({ size }) => {
		return {
			sm: { width: 20, height: 10 },
			md: { width: 40, height: 21 },
			lg: { width: 60, height: 30 },
			xl: { width: 80, height: 40 },
		}[size!];
	},
	({ disabled, theme }) => {
		if (disabled) {
			return {
				borderColor: theme.palette.disabled,
			};
		}
	},
);

const CheckboxSlider = styled(Animated.View)<{
	checked: CheckboxProps['checked'];
	color: CheckboxProps['color'];
	disabled: CheckboxProps['disabled'];
	size: CheckboxProps['size'];
}>(
	() => {
		return {
			borderRadius: 50,
			borderWidth: 1,
		};
	},
	({ theme, color, checked }) => {
		return {
			borderWidth: checked ? 0 : 1,
			backgroundColor: checked
				? theme.palette[color!].main
				: theme.palette.background.main,
		};
	},
	({ size }) => {
		return {
			sm: { width: 8, height: 8 },
			md: { width: 15, height: 15 },
			lg: { width: 24, height: 24 },
			xl: { width: 32, height: 32 },
		}[size!];
	},
	({ disabled, theme }) => {
		if (disabled) {
			return {
				borderColor: theme.palette.disabled,
			};
		}
	},
);

const CheckboxDescription = styled.Text<{
	disabled: CheckboxProps['disabled'];
	size: CheckboxProps['size'];
}>(
	({ theme }) => {
		return {
			fontFamily: theme.fontFamily,
		};
	},
	({ theme, size }) => {
		return {
			fontSize: theme.fontSize[size!],
		};
	},
	({ disabled, theme }) => {
		if (disabled) {
			return {
				color: theme.palette.disabled,
			};
		}
	},
);
