import React, { useEffect, useState } from 'react';
import { format, format as formatFn, parse } from 'date-fns';
import { Platform, StyleProp, View, ViewStyle } from 'react-native';
import { measure, runOnJS, runOnUI, useAnimatedRef } from 'react-native-reanimated';

import { Calendar } from '@procyon/components/Calendar';
import { useModal } from '@procyon/components/Modal/context';
import { CalendarFoldIcon } from '@procyon/icons/CalendarFold';
import { useTheme } from '@procyon/styles/Theme';
import { InputElementType } from '@procyon/types/form';
import { IMask, Masked } from '@procyon/utils/imask';

import { TextField, TextFieldProps } from './TextField';

export const DEFAULT_DATE_FORMAT = 'dd.MM.yyyy';

export type DatePickerProps<V = Date> = Omit<
	{
		format?: string;
		mask?: Masked;
	} & Omit<TextFieldProps, 'onChange'> &
		InputElementType<V>,
	'onFocus' | 'onClick' | 'onBlur'
>;

export const DatePicker: React.FC<DatePickerProps> = ({
	onChange,
	label,
	name,
	value,
	...others
}) => {
	const dateFormat = others.format ?? DEFAULT_DATE_FORMAT;

	const formatValue = (date?: Date) => {
		if (date) return formatFn(date, dateFormat);
		return undefined;
	};

	const { theme } = useTheme();
	const viewRef = useAnimatedRef<View>();
	const [inputValue, setInputValue] = useState<string | undefined>(undefined);

	useEffect(() => {
		if (value) {
			setInputValue(formatFn(value, DEFAULT_DATE_FORMAT));
		}
	}, []);

	const [openModal, closeModal, { opened }] = useModal<{
		rootStyle: StyleProp<ViewStyle>;
	}>(name, {
		Component: ({ rootStyle }) => (
			<View style={[rootStyle]}>
				<Calendar onClick={handleDateClick} value={value} selected={value} />
			</View>
		),
	});

	const handleChangeValue = (e, value: Date) => {
		onChange?.(e, value as any);
		setInputValue(formatValue(value));
	};

	const handleDateClick = (value: Date) => {
		handleChangeValue(undefined, value);
		closeModal();
	};

	const handleOpenCalendar = () => {
		if (opened.includes(name!)) {
			closeModal();
			return;
		}
		runOnUI(() => {
			const layout = measure(viewRef);
			runOnJS(openModal)({
				props: {
					rootStyle: {
						position: 'absolute',
						...(!!layout?.pageY && { top: layout.pageY + 65 }),
						...(!!layout?.pageX && { left: layout.pageX }),
						zIndex: 7,
						margin: theme.spacing.sm,
						marginHorizontal: 0,
						padding: theme.spacing.lg,
						borderRadius: theme.rounded.small,
						backgroundColor: theme.palette.background.main,
						borderWidth: 1,
						borderColor: theme.palette.secondary.main,
					},
				},
			});
		})();
	};

	const mask =
		others.mask ??
		IMask.createMask({
			mask: Date,
			pattern: DEFAULT_DATE_FORMAT,
			blocks: {
				dd: {
					mask: IMask.MaskedRange,
					from: 1,
					to: 31,
				},
				MM: {
					mask: IMask.MaskedRange,
					from: 1,
					to: 12,
				},
				yyyy: {
					mask: IMask.MaskedRange,
					from: 1900,
					to: 9999,
				},
			},
			parse: (value: string) => {
				return parse(value, DEFAULT_DATE_FORMAT, new Date());
			},
			format: (value: Date) => {
				return format(value, DEFAULT_DATE_FORMAT);
			},
			autofix: true,
		});

	return (
		<View ref={viewRef}>
			<TextField
				name={name}
				label={label}
				placeholder={dateFormat}
				LeftIcon={(props) => (
					<View style={[Platform.OS === 'web' && ({ cursor: 'pointer' } as any)]}>
						<CalendarFoldIcon {...props} onClick={() => handleOpenCalendar()} />
					</View>
				)}
				{...others}
				mask={mask}
				value={inputValue ?? ''}
				onChange={(e, value) => {
					if (mask.isComplete) {
						handleChangeValue(e, parse(mask.value, dateFormat, new Date()));
					}
					setInputValue(value ?? undefined);
				}}
			/>
		</View>
	);
};
