import React, { FC, ReactNode } from 'react';
import { StyleProp, View, ViewStyle } from 'react-native';
import { measure, runOnJS, runOnUI, useAnimatedRef } from 'react-native-reanimated';
import invariant from 'tiny-invariant';

import { Button, ButtonProps } from '@procyon/components/Button';
import { ButtonGroup } from '@procyon/components/ButtonGroup';
import { useModal } from '@procyon/components/Modal/context';
import { ChevronDownIcon } from '@procyon/icons/ChevronDown';
import { ChevronUpIcon } from '@procyon/icons/ChevronUp';
import { useTheme } from '@procyon/styles/Theme';
import { debounceAfter } from '@procyon/utils/debounce';

export type DropdownButtons = Partial<ButtonProps>;

export type DropdownProps = {
	Buttons?: FC<DropdownButtons>;
	MainButton: ReactNode;
	name: string;
	size?: 'lg' | 'md' | 'sm' | 'xl';
};

export const Dropdown: FC<DropdownProps> = ({
	Buttons,
	MainButton,
	size = 'md',
	name,
}) => {
	const { theme } = useTheme();
	const viewRef = useAnimatedRef<View>();

	const [openModal, closeModal, { opened }] = useModal<{
		rootStyle: StyleProp<ViewStyle>;
	}>(name, {
		Component: ({ rootStyle }) => {
			return (
				<View style={[rootStyle]}>
					{Buttons && (
						<Buttons
							{...{
								...(size ? { size } : {}),
								fullWidth: true,
								style: {
									container: {
										marginVertical: 3,
									},
								},
							}}
							onClick={debounceAfter(() => handleClose(), 100)}
						/>
					)}
				</View>
			);
		},
	});

	const handleClose = () => {
		closeModal();
	};

	const handleOpenButtons = () => {
		if (opened.includes(name)) {
			closeModal();
			return;
		}
		runOnUI(() => {
			if (!viewRef?.current) {
				invariant('Unable to measure element for obtain position!');
				return null;
			}
			const layout = measure(viewRef);
			runOnJS(openModal)({
				props: {
					rootStyle: {
						position: 'absolute',
						...(!!layout?.width && { width: layout.width }),
						...(!!layout?.pageY && {
							top: layout.pageY + theme.spacing[size] * 2 + theme.fontSize[size] * 2,
						}),
						...(!!layout?.pageX && { left: layout.pageX }),
						zIndex: 7,
					},
				},
				style: {
					overflow: {
						backgroundColor: 'transparent',
					},
				},
			});
		})();
	};

	return (
		<ButtonGroup ref={viewRef}>
			{MainButton}
			{Buttons && (
				<Button
					style={{ container: { minWidth: 40, maxWidth: 50 } }}
					onClick={() => handleOpenButtons()}
					{...(size ? { size } : {})}>
					{opened.includes(name) ? <ChevronUpIcon /> : <ChevronDownIcon />}
				</Button>
			)}
		</ButtonGroup>
	);
};
