import React, { Children, FC, ReactNode } from 'react';
import { range } from 'ramda';
import { Pressable, PressableProps, StyleProp, View, ViewStyle } from 'react-native';

import { useMediaQuery } from '@procyon/styles/useMediaQuery';

export type GridProps = {
	children: ReactNode;
	cols?: Partial<Record<'xs' | 'sm' | 'md' | 'lg' | 'xl', number>>;
	fill?: boolean;
	layout?: Partial<Record<'xs' | 'sm' | 'md' | 'lg' | 'xl', (number | string)[]>>;
	style?: {
		cell?: StyleProp<ViewStyle>;
		row?: StyleProp<ViewStyle>;
	};
} & Omit<ViewStyle, 'direction'> &
	Omit<PressableProps, 'style'>;

export const Grid: FC<GridProps> = ({
	children,
	cols,
	gap = 0,
	layout,
	style,
	...props
}) => {
	const mq = useMediaQuery();
	if (!mq) {
		return null;
	}

	let itemIndex = -1;
	const isMobile = mq === 'xs';
	const childrenInArray = Children.toArray(children);

	const defaultColCount = 1;
	const colCount = isMobile
		? cols?.xs ?? defaultColCount
		: cols?.[mq] ??
			cols?.xl ??
			cols?.lg ??
			cols?.md ??
			cols?.sm ??
			childrenInArray.length;

	const defaultLayout = new Array(childrenInArray.length).fill(1);
	const activeLayout = isMobile
		? layout?.xs ?? defaultLayout
		: layout?.[mq] ??
			layout?.xl ??
			layout?.lg ??
			layout?.md ??
			layout?.sm ??
			defaultLayout;

	const chunks = splitIntoChunks(childrenInArray, colCount, activeLayout);

	return chunks.map((chunk, chunkIndex) => {
		const emptyColsArray = new Array(colCount - chunk.length).fill(0);
		return (
			<Pressable
				key={chunkIndex}
				style={[{ display: 'flex', flexDirection: 'column' }]}
				{...props}>
				<View
					style={[
						{
							display: 'flex',
							flexDirection: 'row',
							cursor: 'auto',
							flex: 1,
						},
						style?.row,
					]}>
					{chunk.map((child, childIndex) => {
						itemIndex++;
						const itemLayout = activeLayout.at(itemIndex);
						const span = typeof itemLayout === 'string' ? 1 : itemLayout ?? 1;
						range(0, span - 1).forEach(() => {
							emptyColsArray.pop();
						});
						return (
							<View
								key={childIndex}
								style={[
									{
										flexGrow: isMobile ? 1 : typeof itemLayout === 'string' ? 0 : span,
										flexBasis: isMobile
											? 'auto'
											: typeof itemLayout === 'string'
												? (itemLayout as unknown as number)
												: 1,
									},
									style?.cell,
								]}>
								<View style={{ marginRight: childIndex !== chunk.length - 1 ? gap : 0 }}>
									{child}
								</View>
							</View>
						);
					})}
					{emptyColsArray.map((n, i) => (
						<View style={{ flex: 1 }} key={i} />
					))}
				</View>
			</Pressable>
		);
	});
};

function splitIntoChunks(
	array: ReactNode[],
	colCount: number,
	layout: (number | string)[],
) {
	const result: ReactNode[][] = [];
	let chunk: ReactNode[] = [];
	let counter = 0;

	for (let i = 0; i < array.length; i++) {
		const item = array[i];
		const itemSize: number = typeof layout[i] === 'string' ? 1 : (layout[i] as number);

		if (counter + itemSize > colCount) {
			result.push(chunk);
			chunk = [item];
			counter = itemSize;
		} else {
			chunk.push(item);
			counter += itemSize;
		}

		if (counter === colCount) {
			result.push(chunk);
			chunk = [];
			counter = 0;
		}
	}

	if (chunk.length) {
		result.push(chunk);
	}

	return result;
}
