import React, { FC } from 'react';
import styled from '@emotion/native';
import {
	FlatList,
	GestureResponderEvent,
	StyleProp,
	View,
	ViewStyle,
} from 'react-native';

import { ListItem, ListItemProps } from '@procyon/components/ListItem';

export type ListData = {
	id: number | string;
	label?: string;
};

type CommonProps<Items extends ListData[]> = {
	Item?: FC<ListItemProps<Items[0] & ListData, 'view' | undefined>>;
	color?: 'primary' | 'secondary' | 'positive' | 'negative';
	compact?: boolean;
	data?: Items;
	getLabel?: (items: Items[0]) => string;
	selected?: ListData['id'];
	size?: 'sm' | 'md' | 'lg' | 'xl';
	style?: {
		container?: StyleProp<ViewStyle>;
		list?: StyleProp<ViewStyle>;
	};
};

export type OnClickProp = {
	onClick: (event: GestureResponderEvent, id: ListData['id']) => void;
};

export type InteractiveMode<Items extends ListData[]> = {
	mode?: undefined;
} & OnClickProp &
	CommonProps<Items>;

export type ViewMode<Items extends ListData[]> = {
	mode: 'view';
} & Partial<OnClickProp> &
	CommonProps<Items>;

export type ListProps<Items extends ListData[]> =
	| InteractiveMode<Items>
	| ViewMode<Items>;

export const List = <Items extends ListData[]>({
	style,
	data,
	selected,
	getLabel,
	Item = ListItem,
	mode,
	...props
}: ListProps<Items>) => {
	return (
		<ListContainer style={[style?.container]}>
			<FlatList
				data={data}
				style={[style?.list]}
				renderItem={({ item }) => (
					<Item
						{...props}
						onClick={props.onClick as any}
						mode={mode as any}
						item={item}
						selected={item.id === selected}
						style={{
							container: [style?.list],
						}}>
						{item.label ? item.label : getLabel?.(item)}
					</Item>
				)}
				keyExtractor={(item) => item.id.toString()}
			/>
		</ListContainer>
	);
};

const ListContainer = styled(View)(({ theme }) => {
	return {
		borderRadius: theme.rounded.small,
		backgroundColor: theme.palette.background.main,
		margin: theme.spacing.sm,
		padding: theme.spacing.sm,
		borderWidth: 1,
		borderColor: theme.palette.secondary.light,
	};
});
