import React, { ReactNode, useMemo, useState } from 'react';
import styled from '@emotion/native';
import { GestureResponderEvent, StyleProp, Text, View, ViewProps } from 'react-native';
import { useAnimatedRef } from 'react-native-reanimated';
import Values from 'values.js';

import { ListData } from '@procyon/components/List';
import { RippleEffect } from '@procyon/components/RippleEffect';
import { useTheme } from '@procyon/styles/Theme';
import { isDark } from '@procyon/styles/utils/color';
import { debounceAfter } from '@procyon/utils/debounce';

type CommonProps<Item extends ListData> = {
	children: ReactNode;
	color?: 'primary' | 'secondary' | 'positive' | 'negative';
	compact?: boolean;
	item: Item;
	selected?: boolean;
	size?: 'sm' | 'md' | 'lg' | 'xl';
	style?: {
		container?: StyleProp<ViewProps>;
	};
};

type OnClick = (event: GestureResponderEvent, id: ListData['id']) => void;

type InteractiveMode<Item extends ListData, Mode = undefined> = {
	mode?: Mode;
	onClick: OnClick;
} & CommonProps<Item>;

type ViewMode<Item extends ListData, Mode = 'view'> = {
	mode: Mode;
	onClick?: OnClick;
} & CommonProps<Item>;

export type ListItemProps<Item extends ListData, Mode extends 'view' | undefined> =
	| InteractiveMode<Item, Mode>
	| ViewMode<Item, Mode>;

export const ListItem = <Item extends ListData, Mode extends 'view' | undefined>({
	style,
	size = 'md',
	mode,
	onClick,
	children,
	color = 'primary',
	selected,
	item,
}: ListItemProps<Item, Mode>) => {
	const rippleAnimationRef = useAnimatedRef<View>();

	const [hover, setHover] = useState(false);

	const { theme } = useTheme();

	const handleClick = debounceAfter((event: GestureResponderEvent) => {
		onClick?.(event, item.id);
	}, 100);

	const rippleColor = useMemo(() => {
		const val = theme.palette[color!].main;
		if (isDark(val)) {
			return new Values(val).tint(30).rgbString();
		} else {
			return new Values(val).shade(30).rgbString();
		}
	}, [color]);

	return (
		<ListItemContainer
			aria-label="list-item"
			disabled={mode === 'view'}
			onHoverIn={() => setHover(true)}
			onHoverOut={() => setHover(false)}
			color={color}
			hover={hover}
			selected={selected}
			style={[style?.container]}
			onPress={(event) => handleClick(event)}
			ref={rippleAnimationRef}>
			<RippleEffect
				ref={rippleAnimationRef}
				color={rippleColor}
				style={{
					container: {
						padding: theme.spacing.md,
						borderRadius: theme.rounded.small,
					},
				}}>
				{typeof children === 'string' ? (
					<ListItemView>
						<ListItemText color={color} size={size} selected={selected} hover={hover}>
							{children}
						</ListItemText>
					</ListItemView>
				) : (
					children
				)}
			</RippleEffect>
		</ListItemContainer>
	);
};

const ListItemContainer = styled.Pressable<{
	color: ListItemProps<any, any>['color'];
	hover: boolean;
	selected: ListItemProps<any, any>['selected'];
}>(({ theme, color, hover, selected }) => {
	return {
		backgroundColor: theme.palette.background.main,
		borderRadius: theme.rounded.small,
		marginVertical: 1,

		...(hover && {
			backgroundColor: theme.palette[color!].light,
			...(color === 'primary' && {
				backgroundColor: theme.palette.highlight.light,
			}),
		}),

		...(selected && {
			backgroundColor: theme.palette[color!].light,
		}),
	};
});

export const ListItemView = styled(View)(() => {
	return {
		display: 'flex',
		flexDirection: 'row',
		alignItems: 'center',
	};
});

export const ListItemText = styled(Text)<{
	color?: ListItemProps<any, any>['color'];
	hover?: boolean;
	selected?: ListItemProps<any, any>['selected'];
	size?: ListItemProps<any, any>['size'];
}>(({ theme, size = 'md', color = 'primary', selected, hover }) => {
	return {
		fontFamily: theme.fontFamily,
		paddingHorizontal: theme.spacing.sm,
		fontSize: theme.fontSize[size!],

		...(color === 'primary' && {
			color: theme.palette.text.main,
		}),
		...(hover && {
			color: theme.palette.highlight.dark,
		}),
		...(selected && {
			color: theme.palette[color!].dark,
		}),
		...(color === 'secondary' && {
			color: theme.palette.secondary.dark,
		}),
		...(color === 'negative' && {
			color: theme.palette.negative.dark,
		}),
		...(color === 'positive' && {
			color: theme.palette.positive.dark,
		}),
	};
});
