import React, { FC } from 'react';
import styled from '@emotion/native';
import { StyleProp, TextStyle, View, ViewStyle } from 'react-native';
import { useAnimatedRef } from 'react-native-reanimated';

import { RippleEffect } from '@procyon/components/RippleEffect';
import { ExternalLinkIcon } from '@procyon/icons/ExternalLink';
import { useTheme } from '@procyon/styles/Theme';

export type MenuItemProps = {
	Icon?: FC<{
		size: number;
	}>;
	active?: boolean;
	children?: string;
	external?: boolean;
	href?: string;
	onNav?: (item: Omit<MenuItemProps, 'onNav'>) => void;
	side?: boolean;
	style?: {
		container?: StyleProp<ViewStyle>;
		text?: StyleProp<TextStyle>;
	};
};

export const MenuItem: FC<MenuItemProps> = ({ onNav, Icon, ...item }) => {
	const { theme } = useTheme();
	const rippleAnimationRef = useAnimatedRef<View>();

	return (
		<MenuItemContainer
			side={item.side}
			style={[item.style?.container]}
			active={item.active}
			onPress={() => onNav?.(item)}
			ref={rippleAnimationRef}>
			<RippleEffect
				ref={rippleAnimationRef}
				style={{
					container: {
						display: 'flex',
						flexDirection: 'row',
						alignItems: 'center',
						gap: 2,
						borderRadius: theme.spacing.md,
						padding: theme.spacing.md,
					},
				}}
				color={
					item.active
						? theme.palette[item.side ? 'secondary' : 'primary'].light
						: theme.palette[item.side ? 'secondary' : 'primary'].dark
				}>
				{item.side && !Icon ? (
					<View style={{ width: theme.fontSize.md }} />
				) : (
					Icon && <Icon size={theme.fontSize.md} />
				)}
				<MenuItemText side={item.side} style={[item.style?.text]} active={item.active}>
					{item.children}
				</MenuItemText>
				{item.external && <ExternalLinkIcon size={theme.fontSize.md} strokeWidth={1} />}
			</RippleEffect>
		</MenuItemContainer>
	);
};

const MenuItemContainer = styled.Pressable<{
	active: MenuItemProps['active'];
	side?: MenuItemProps['side'];
}>(({ theme, active, side }) => {
	return {
		borderRadius: theme.spacing.md,
		backgroundColor: active
			? theme.palette[side ? 'secondary' : 'primary'].dark
			: 'transparent',
		color: active ? theme.palette.primary.light : theme.palette.primary.dark,
	};
});

const MenuItemText = styled.Text<{
	active: MenuItemProps['active'];
	side?: MenuItemProps['side'];
}>(({ theme, active, side }) => {
	return {
		fontFamily: theme.fontFamily,
		fontSize: theme.fontSize.md,
		color: active
			? theme.palette[side ? 'secondary' : 'primary'].light
			: theme.palette[side ? 'secondary' : 'primary'].dark,
	};
});
