import React, { useEffect } from 'react';
import styled from '@emotion/native';
import { Modal, Platform, Pressable, Text, View } from 'react-native';
import Values from 'values.js';

import { notNil } from '@procyon/utils/nil';

import { ModalId, useModal } from './context';

export const Modals = () => {
	const [, , { modals, opened, closeModal, rootRef }] = useModal();

	const handleCloseModal = (modalId: ModalId) => {
		modals[modalId]?.props?.onClose?.();
		closeModal(modalId);
	};

	const handleKeyUp = (e: KeyboardEvent): void => {
		if (e.key === 'Escape') {
			const modalId = opened.at(-1);
			if (modalId !== undefined) {
				handleCloseModal(modalId);
			}
		}
	};

	useEffect(() => {
		if (Platform.OS === 'web') {
			window.addEventListener('keydown', handleKeyUp);
			return () => {
				window.removeEventListener('keydown', handleKeyUp);
			};
		}
	}, [opened]);

	if (!notNil(opened)) {
		return null;
	}

	return opened.map((modalId) => {
		const { id, style, Component = View, props } = modals[modalId] ?? {};
		if (!id) return null;
		return (
			<Modal transparent visible key={id} ref={rootRef}>
				<ModalOverflow style={[style?.overflow]} onPressIn={() => handleCloseModal(id)} />
				<Component {...props} />
			</Modal>
		);
	});
};

const ModalOverflow = styled(Pressable)(({ theme }) => {
	return {
		...(Platform.OS === 'web' && ({ cursor: 'close' } as any)),
		position: 'absolute',
		top: 0,
		right: 0,
		bottom: 0,
		left: 0,
		zIndex: 6,
		backgroundColor: `${new Values(theme.palette.background.dark).hexString()}80`,
	};
});

export const ModalContainer = styled(Pressable)(({ theme }) => {
	return {
		width: 200,
		marginLeft: 'auto',
		marginRight: 'auto',
		marginTop: 'auto',
		marginBottom: 'auto',
		zIndex: 7,
		padding: theme.spacing.lg,
		backgroundColor: theme.palette.background.main,
		borderRadius: theme.spacing.md,
	};
});

export const ModalTitle = styled(Text)(({ theme }) => {
	return {
		fontSize: theme.fontSize.lg,
		fontFamily: theme.fontFamily,
		borderBottomWidth: 1,
		borderBottomColor: theme.palette.primary.dark,
		paddingHorizontal: theme.spacing.md,
		paddingBottom: theme.spacing.md,
		marginBottom: 10,
	};
});
