import React, { FC, useMemo, useState } from 'react';
import styled from '@emotion/native';
import { View } from 'react-native';
import { Gesture, GestureDetector } from 'react-native-gesture-handler';
import Animated, { useAnimatedStyle, useSharedValue } from 'react-native-reanimated';

import { Label } from '@procyon/components/Label';
import { useTranslate } from '@procyon/localization/tolgee';

export type PuzzleProps = {
	ballDimension?: {
		height: number;
		width: number;
	};
	containerDimension?: {
		height: number;
		width: number;
	};
	holeDimension?: {
		height: number;
		width: number;
	};
	onVerified: () => void;
};

function clamp(val, min, max) {
	return Math.min(Math.max(val, min), max);
}

function getRandomPosition(min: number, max: number) {
	min = Math.ceil(min);
	max = Math.floor(max);
	return Math.floor(Math.random() * (max - min + 1)) + min;
}

export const Puzzle: FC<PuzzleProps> = ({
	onVerified,
	containerDimension = {
		width: 200,
		height: 100,
	},
	ballDimension = {
		width: 20,
		height: 20,
	},
	holeDimension = {
		width: 30,
		height: 30,
	},
}) => {
	const [verified, setVerified] = useState(false);

	const { t } = useTranslate();

	const playgroundWidth = containerDimension.width / 2;

	const translationX = useSharedValue(getRandomPosition(0, playgroundWidth - 20));
	const translationY = useSharedValue(
		getRandomPosition(0, containerDimension.height - 20),
	);

	const prevTranslationX = useSharedValue(0);
	const prevTranslationY = useSharedValue(0);

	const [holeX, holeY] = useMemo(() => {
		return [
			getRandomPosition(playgroundWidth, containerDimension.width - 30),
			getRandomPosition(0, containerDimension.height - 30),
		];
	}, []);

	const animatedStyles = useAnimatedStyle(() => ({
		transform: [{ translateX: translationX.value }, { translateY: translationY.value }],
	}));

	const panGesture = Gesture.Pan()
		.enabled(!verified)
		.minDistance(0)
		.onStart(() => {
			prevTranslationX.value = translationX.value;
			prevTranslationY.value = translationY.value;
		})
		.onUpdate((event) => {
			const maxTranslateX = containerDimension.width - ballDimension.width;
			const maxTranslateY = containerDimension.height - ballDimension.height;

			translationX.value = clamp(
				prevTranslationX.value + event.translationX,
				0,
				maxTranslateX,
			);
			translationY.value = clamp(
				prevTranslationY.value + event.translationY,
				0,
				maxTranslateY,
			);
		})
		.onFinalize(() => {
			const roundHoleX = Math.floor(holeX);
			const roundTranslationX = Math.floor(translationX.value);
			const roundHoleY = Math.floor(holeY);
			const roundTranslationY = Math.floor(translationY.value);

			if (
				roundTranslationX >= roundHoleX &&
				roundTranslationX <= roundHoleX + 10 &&
				roundTranslationY >= roundHoleY &&
				roundTranslationY <= roundHoleY + 10
			) {
				setVerified(true);
				onVerified();
			}
		});

	return (
		<>
			<Label>{t('procyon.puzzle.manual')}</Label>
			<GestureDetector gesture={panGesture}>
				<PuzzleContainer style={[containerDimension]}>
					<PuzzleHole verified={verified} x={holeX} y={holeY} style={[holeDimension]} />
					{!verified && <PuzzleBall style={[ballDimension, animatedStyles]} />}
				</PuzzleContainer>
			</GestureDetector>
		</>
	);
};

const PuzzleContainer = styled(View)(({ theme }) => {
	return {
		position: 'relative',
		borderWidth: 1,
		borderColor: theme.palette.secondary.main,
		borderRadius: theme.rounded.small,
	};
});

const PuzzleBall = styled(Animated.View)(({ theme }) => {
	return {
		borderRadius: theme.rounded.full,
		position: 'absolute',
		...({ cursor: 'move' } as any),
		borderWidth: 1,
		borderColor: theme.palette.primary.dark,
		backgroundColor: theme.palette.primary.light,
	};
});

const PuzzleHole = styled(Animated.View)<{
	verified: boolean;
	x: number;
	y: number;
}>(({ theme, x, y, verified }) => {
	return {
		borderRadius: theme.rounded.full,
		borderWidth: 2,
		borderStyle: 'dashed',
		borderColor: theme.palette.negative.main,
		transform: [{ translateX: x }, { translateY: y }],

		...(verified && {
			backgroundColor: theme.palette.positive.main,
			borderColor: theme.palette.primary.dark,
			borderStyle: 'solid',
		}),
	};
});
