import React, { Children, FC, isValidElement, ReactNode, useMemo } from 'react';
import styled from '@emotion/native';
import { View } from 'react-native';

import { Label } from '@procyon/components/Label';
import { RadioItem } from '@procyon/components/RadioItem';
import { useMediaQuery } from '@procyon/styles/useMediaQuery';
import { InputElementType } from '@procyon/types/form';

import { FieldMessage } from './FieldMessage';

export type RadioButtonProps = Omit<
	{
		children: ReactNode;
		horizontal?: boolean;
	} & InputElementType<string>,
	'onFocus' | 'onBlur' | 'onClick'
>;

const findRadioItems = (children: ReactNode): ReactNode => {
	return Children.map(Children.toArray(children), (child: ReactNode) => {
		const elementChild = child as ReactNode;
		if (isValidElement(elementChild)) {
			if (elementChild.type === RadioItem) {
				return elementChild;
			} else {
				return findRadioItems(elementChild.props.children);
			}
		}
		return null;
	});
};

export const RadioButton: FC<RadioButtonProps> = ({
	disabled,
	label,
	size,
	labelSize,
	color = 'primary',
	value,
	horizontal,
	message,
	children,
	fullWidth,
	...props
}) => {
	const mq = useMediaQuery();
	const radioItems = useMemo(() => findRadioItems(children), [children]);

	return (
		<View
			style={[
				{
					width: fullWidth || (fullWidth === undefined && mq === 'xs') ? '100%' : 'auto',
				},
			]}
			aria-label="radio-button">
			{label && (
				<Label disabled={disabled} size={labelSize}>
					{label}
				</Label>
			)}
			<RadioButtonOptions horizontal={horizontal}>
				{Children.map(radioItems, (child, index) => {
					if (isValidElement(child) && child.type === RadioItem) {
						return {
							...child,
							props: {
								color,
								size,
								...props,
								...child.props,
								selected: child.props.value === value,
								disabled: disabled || child.props.disabled,
								style: {
									container: {
										...(index === Children.count(children) - 1 && {
											borderWidth: 0,
										}),
									},
								},
							},
						};
					}
					return child;
				})}
			</RadioButtonOptions>
			<FieldMessage message={message} size={size} color={color} />
		</View>
	);
};

const RadioButtonOptions = styled(View)<{
	horizontal: RadioButtonProps['horizontal'];
}>(({ theme, horizontal }) => {
	return {
		display: 'flex',
		flexDirection: 'column',
		gap: theme.spacing.sm,
		marginTop: theme.spacing.md - 1,

		...(horizontal && {
			flexDirection: 'row',
			gap: theme.spacing.xl,
		}),
	};
});
