import React, { FC, useMemo } from 'react';
import styled from '@emotion/native';
import { Platform, Pressable, StyleProp, Text, View, ViewStyle } from 'react-native';
import { measure, runOnJS, runOnUI, useAnimatedRef } from 'react-native-reanimated';

import { List, ListData, ListProps } from '@procyon/components/List';
import { useModal } from '@procyon/components/Modal/context';
import { ChevronDownIcon } from '@procyon/icons/ChevronDown';
import { InputElementType } from '@procyon/types/form';

import { TextField, TextFieldProps } from './TextField';

export type ItemWithLabel = {
	id: ListData['id'];
	label: string;
};

export type ItemWithoutLabel = {
	id: ListData['id'];
};

type GetLabel<Item extends ListData> = { getLabel: (item: Item) => string };

export type SelectboxProps<Items extends ItemWithLabel[] | ItemWithoutLabel[]> = Omit<
	TextFieldProps,
	'onChange' | 'style'
> & {
	Input?: FC<TextFieldProps & { data: Items }>;
	Item?: ListProps<Items>['Item'];
	clearable?: boolean;
	data?: Items;
	style?: {
		container?: StyleProp<ViewStyle>;
		input?: TextFieldProps['style'];
		list?: ListProps<any>['style'];
	};
} & InputElementType<string> &
	(Items extends ItemWithLabel[] ? Partial<GetLabel<Items[0]>> : GetLabel<Items[0]>);

export const Selectbox = <Items extends ListData[]>(props: SelectboxProps<Items>) => {
	const {
		value,
		onChange,
		label,
		message,
		name,
		data,
		Input = DefaultInput,
		Item,
		getLabel,
		style,
		...others
	} = props;
	const viewRef = useAnimatedRef<View>();

	const longestLabelLength = useMemo(() => {
		const lengths: number[] = [];
		data?.forEach((item) => {
			const label = item?.label ?? getLabel(item);
			lengths.push(label.length);
		});
		return Math.max(...lengths);
	}, [data]);

	const [openModal, closeModal, { opened }] = useModal<{
		rootStyle: StyleProp<ViewStyle>;
	}>(name, {
		Component: ({ rootStyle }) => {
			return (
				<View style={[rootStyle]}>
					<List
						Item={Item}
						selected={value}
						data={data}
						getLabel={getLabel}
						onClick={(event, id: string) => {
							event.preventDefault();
							onChange?.(event, id);
							closeModal();
						}}
						style={{
							container: [
								{
									marginHorizontal: 0,
									marginBottom: 10,
								},
								style?.list?.container,
							],
							list: [
								{
									maxHeight: 300,
								},
								style?.list?.list,
							],
						}}
					/>
				</View>
			);
		},
	});

	const handleOpenOptions = () => {
		if (opened.includes(name!)) {
			closeModal();
			return;
		}
		runOnUI(() => {
			const layout = measure(viewRef);
			runOnJS(openModal)({
				props: {
					rootStyle: {
						position: 'absolute',
						...(!!layout?.width && { width: layout.width }),
						...(!!layout?.pageY && { top: layout.pageY + 65 }),
						...(!!layout?.pageX && { left: layout.pageX }),
						zIndex: 7,
					},
				},
			});
		})();
	};

	const width = Math.min(300, longestLabelLength * 8 + 60);

	return (
		<View
			style={[{ position: 'relative', minWidth: width }, style?.container]}
			ref={viewRef}>
			<TextField
				{...others}
				style={{
					wrapper: {
						columnGap: 0,
					},
				}}
				label={label}
				onBlur={(e) => {
					others.onBlur?.(e, value);
				}}
				onFocus={(e) => {
					others.onFocus?.(e, value);
				}}
				Input={(props) => (
					<Input
						{...props}
						style={{
							...style?.input,
							input: [style?.input?.input, { paddingHorizontal: 10 }],
						}}
						data={data as any}
						getLabel={getLabel}
						value={value?.toString()}
					/>
				)}
				message={message}
				onClick={handleOpenOptions}
				RightIcon={(props) => (
					<Pressable
						onPressIn={handleOpenOptions}
						style={[Platform.OS === 'web' && { cursor: 'pointer' }]}>
						<ChevronDownIcon {...props} />
					</Pressable>
				)}
			/>
		</View>
	);
};

const DefaultInput = <Items extends ListData[]>({
	value,
	data,
	getLabel,
}: TextFieldProps & {
	data?: SelectboxProps<Items>['data'];
	getLabel?: SelectboxProps<Items>['getLabel'];
}) => {
	const item = data?.find((item) => item.id.toString() === value && !!value);
	if (!item) return null;
	return <FieldText>{getLabel ? getLabel(item) : item.label}</FieldText>;
};

export const FieldText = styled(Text)<{
	size?: SelectboxProps<any>['size'];
}>(({ theme, size = 'md' }) => {
	return {
		...(Platform.OS === 'web' && { cursor: 'pointer' }),
		color: theme.palette.text.main,
		fontFamily: theme.fontFamily,
		fontSize: theme.fontSize[size],
	};
});
