import React, {
	Children,
	createContext,
	FC,
	isValidElement,
	ReactNode,
	useContext,
} from 'react';
import styled from '@emotion/native';
import { Text, View } from 'react-native';

import { GridProps } from '@procyon/components/Grid';
import { Stack } from '@procyon/components/Stack';

type TableContext = {
	cols?: GridProps['cols'];
	hoverable?: boolean;
	layout?: GridProps['layout'];
};

const TableContext = createContext<TableContext | undefined>(undefined);

export type TableProps = {
	children: ReactNode[];
	cols?: GridProps['cols'];
	hoverable?: boolean;
	layout?: GridProps['layout'];
};

export const Table: FC<TableProps> = ({ cols, layout, hoverable = false, children }) => {
	const childCount = React.Children.count(children);

	return (
		<TableContext.Provider value={{ cols, layout, hoverable }}>
			<TableContainer direction="column">
				{Children.map(children, (child, i) => {
					if (i === childCount - 1) {
						if (isValidElement(child)) {
							return {
								...child,
								props: {
									...child.props,
									last: true,
								},
							};
						}
					}
					return child;
				})}
			</TableContainer>
		</TableContext.Provider>
	);
};

export const useTable = () => {
	const context = useContext(TableContext);
	if (!context) {
		throw new Error('useTable must be used within a TableProvider!');
	}
	return context;
};

const TableContainer = styled(Stack)(({ theme }) => {
	return {
		borderRadius: theme.rounded.small,
	};
});

export const TableHead = styled(View)(({ theme }) => {
	return {
		justifyContent: 'center',
		padding: theme.spacing.md,
	};
});

export const TableCell = styled(View)(({ theme }) => {
	return {
		justifyContent: 'center',
		padding: theme.spacing.md,
	};
});

export const TableText = styled(Text)<{
	head?: boolean;
}>(({ theme, head }) => {
	return {
		fontFamily: theme.fontFamily,
		fontSize: theme.fontSize.md,
		...(head
			? {
					color: theme.palette.secondary.light,
				}
			: {
					color: theme.palette.text.main,
				}),
	};
});
