import React, { FC } from 'react';
import styled from '@emotion/native';
import {
	InputModeOptions,
	Pressable,
	StyleProp,
	TextInputFocusEventData,
	TextStyle,
	View,
	ViewStyle,
} from 'react-native';
import { TextInputChangeEventData } from 'react-native/Libraries/Components/TextInput/TextInput';
import { NativeSyntheticEvent } from 'react-native/Libraries/Types/CoreEventTypes';
import Values from 'values.js';

import { FieldMessage } from '@procyon/components/FieldMessage';
import { useTheme } from '@procyon/styles/Theme';
import { useMediaQuery } from '@procyon/styles/useMediaQuery';
import { InputElementType } from '@procyon/types/form';
import { Masked } from '@procyon/utils/imask';

import { Label } from './Label';
import { Skeleton } from './Skeleton';

export type TextFieldProps<V = string> = {
	LeftIcon?: FC<any>;
	RightIcon?: FC<any>;
	autoFocus?: boolean;
	fullWidth?: boolean;
	leftText?: string;
	mask?: Masked;
	mode?: InputModeOptions;
	multiline?: boolean;
	readonly?: boolean;
	rightText?: string;
	rows?: number;
	secret?: boolean;
	style?: {
		container?: StyleProp<ViewStyle>;
		icon?: StyleProp<ViewStyle | TextStyle>;
		input?: StyleProp<TextStyle>;
		label?: StyleProp<TextStyle>;
		message?: StyleProp<TextStyle>;
		wrapper?: StyleProp<ViewStyle>;
	};
} & InputElementType<V>;

export const TextField: React.FC<
	TextFieldProps & {
		Input?: FC<TextFieldProps>;
	}
> = (props) => {
	const {
		Input = TextFieldInput,
		mode = 'text',
		onClick,
		onFocus,
		onBlur,
		onChange,
		disabled = false,
		name,
		label,
		color = 'secondary',
		autoFocus,
		mask,
		placeholder,
		multiline,
		LeftIcon,
		RightIcon,
		leftText,
		rightText,
		message,
		style,
		readonly,
		size = 'md',
		labelSize,
		secret,
		loading,
		fullWidth,
		rows,
	} = props;
	const mq = useMediaQuery();
	const { theme } = useTheme();

	const applyMask = (val: typeof props.value) => {
		if (mask && val) {
			mask.resolve(val);
			return mask.value;
		}
		return val;
	};

	const value = applyMask(props.value);

	const handleClick = (e: NativeSyntheticEvent<PointerEvent>) => {
		onClick?.(e);
	};

	const handleFocus = (e: NativeSyntheticEvent<TextInputFocusEventData>) => {
		onFocus?.(e, e.nativeEvent.text);
	};

	const handleBlur = (e: NativeSyntheticEvent<TextInputFocusEventData>): void => {
		onBlur?.(e, e.nativeEvent.text);
	};

	const handleChange = (e: NativeSyntheticEvent<TextInputChangeEventData>): void => {
		const value = e.nativeEvent.text;
		const maskedValue = applyMask(value);
		if (onChange) {
			onChange(e, maskedValue);
		}
	};

	return (
		<View
			style={[
				{
					width: fullWidth || (fullWidth === undefined && mq === 'xs') ? '100%' : 'auto',
				},
				style?.container,
			]}
			aria-label="textfield">
			<Label disabled={disabled} size={labelSize}>
				{label}
			</Label>
			<Skeleton active={loading || false} layout={[{ key: '1', height: 40 }]}>
				<TextFieldWrapper color={color} disabled={disabled} style={[style?.wrapper]}>
					{LeftIcon && (
						<TextFieldIconContainer>
							<LeftIcon style={[style?.icon]} color={theme.palette.secondary.dark} />
						</TextFieldIconContainer>
					)}
					{leftText && (
						<TextFieldIconContainer>
							<TextFieldText color={color} size={size}>
								{leftText}
							</TextFieldText>
						</TextFieldIconContainer>
					)}
					<Pressable
						onPress={(e: any) => handleClick(e)}
						style={[
							{
								flex: 1,
								justifyContent: 'center',
							},
						]}>
						<Input
							size={size}
							style={[style?.input]}
							id={name}
							disabled={disabled}
							secureTextEntry={secret}
							inputMode={mode}
							onFocus={handleFocus}
							onBlur={handleBlur}
							onChange={handleChange}
							autoFocus={autoFocus}
							readOnly={readonly || disabled}
							value={value ?? ''}
							multiline={multiline}
							rows={rows}
							placeholder={placeholder}
							placeholderTextColor={new Values(theme.palette.text.light)
								.shade(30)
								.rgbString()}
							autoComplete="off"
						/>
					</Pressable>
					{RightIcon && (
						<TextFieldIconContainer>
							<RightIcon style={[style?.icon]} color={theme.palette.secondary.dark} />
						</TextFieldIconContainer>
					)}
					{rightText && (
						<TextFieldIconContainer>
							<TextFieldText color={color} size={size}>
								{rightText}
							</TextFieldText>
						</TextFieldIconContainer>
					)}
				</TextFieldWrapper>
			</Skeleton>
			<FieldMessage message={message} color={color} size={size} />
		</View>
	);
};

const TextFieldWrapper = styled(View)<{
	color: TextFieldProps['color'];
	disabled: boolean;
}>(({ theme, color, disabled }) => {
	return {
		minHeight: 40,
		display: 'flex',
		flexDirection: 'row',
		columnGap: 10,
		borderRadius: theme.rounded.small,
		paddingHorizontal: theme.spacing.lg,
		borderWidth: 1,
		borderColor: theme.palette[color!].dark,
		backgroundColor: theme.palette[color!].light,

		...(color === 'secondary' && {
			backgroundColor: theme.palette.background.main,
		}),

		...(disabled && {
			borderColor: theme.palette.disabled,
		}),
	};
});

const TextFieldInput = styled.TextInput<{
	disabled: TextFieldProps['disabled'];
	size: TextFieldProps['size'];
}>(({ theme, size, disabled }) => {
	return {
		color: theme.palette.text.main,
		fontFamily: theme.fontFamily,
		paddingVertical: theme.spacing.md,
		fontSize: theme.fontSize[size!],

		...(disabled && {
			color: theme.palette.disabled,
		}),
	};
});

const TextFieldIconContainer = styled(View)(() => {
	return {
		justifyContent: 'center',
	};
});

const TextFieldText = styled.Text<{
	color: TextFieldProps['color'];
	size: TextFieldProps['size'];
}>(({ theme, color, size }) => {
	return {
		color: theme.palette[color!].main,
		fontSize: theme.fontSize[size!],
		fontFamily: theme.fontFamily,
	};
});
