import React, { FC, useEffect, useState } from 'react';
import { format, isDate, parse, setHours, setMinutes, setSeconds } from 'date-fns';
import { format as formatFn } from 'date-fns/format';

import { InputElementType } from '@procyon/types/form';
import { IMask, Masked } from '@procyon/utils/imask';

import { TextField, TextFieldProps } from './TextField';

const DEFAULT_TIME_FORMAT = 'HH:mm';

export type TimePickerProps<V = Date> = Omit<
	{
		hour?: {
			max?: number;
			min?: number;
		};
		mask?: Masked;
		minute?: {
			max?: number;
			min?: number;
		};
	} & Omit<TextFieldProps, 'onChange'> &
		InputElementType<V>,
	'onFocus' | 'onClick' | 'onBlur'
>;

export const TimePicker: FC<TimePickerProps> = ({
	onChange,
	label,
	name,
	hour,
	minute,
	value,
	...others
}) => {
	const [inputValue, setInputValue] = useState<string | undefined>(undefined);

	useEffect(() => {
		if (value) {
			setInputValue(isDate(value) ? formatFn(value, DEFAULT_TIME_FORMAT) : value);
		}
	}, []);

	const mask =
		others.mask ??
		IMask.createMask({
			mask: Date,
			pattern: DEFAULT_TIME_FORMAT,
			blocks: {
				HH: {
					mask: IMask.MaskedRange,
					from: hour?.min ?? 0,
					to: hour?.max ?? 23,
				},
				mm: {
					mask: IMask.MaskedRange,
					from: minute?.min ?? 0,
					to: minute?.max ?? 59,
				},
			},
			parse: (value: string) => {
				return parse(value, DEFAULT_TIME_FORMAT, new Date());
			},
			format: (value: Date) => {
				return format(value, DEFAULT_TIME_FORMAT);
			},
			autofix: true,
		});

	return (
		<TextField
			name={name}
			label={label}
			placeholder={DEFAULT_TIME_FORMAT}
			mode="numeric"
			{...others}
			mask={mask}
			value={inputValue}
			onChange={(e, newValue) => {
				if (mask.isComplete && !!onChange) {
					let changed = value ?? new Date();
					changed = setHours(changed, parseInt(mask.value.split(':')[0]));
					changed = setMinutes(changed, parseInt(mask.value.split(':')[1]));
					changed = setSeconds(changed, 0);
					onChange(e, changed);
				}
				setInputValue(newValue ?? undefined);
			}}
		/>
	);
};
